#ifndef _C_LOADRUN_CTR_
#define _C_LOADRUN_CTR_

namespace kmc {
namespace dcnt {
namespace CTR {

typedef struct {
	DWORD id;
	HANDLE hCmd;
	HANDLE hLog;
	HANDLE hEvt;
	DWORD pid;
} DbgCtrl;

#define DEBUGGER_CTRS		(0)
#define DEBUGGER_CTRSEN		(1)
#define DEBUGGER_CTRJ		(2)
#define DEBUGGER_KINDMAX	(3)

typedef int (*PLRCTR_id2serialno)(DWORD, char*);
typedef int (*PLRCTR_serialno2id)(const char*, DWORD*);
typedef const char* (*PLRCTR_getDefaultBinDir)(int);
typedef const char* (*PLRCTR_getDefaultCfgDir)(int);
typedef int (*PLRCTR_setDir)(const char*, const char*, int);
typedef int (*PLRCTR_setAllocator)(void* (*)(size_t));
typedef int (*PLRCTR_getList)(DWORD**);
typedef int (*PLRCTR_ConnectDebuggerEx)(DWORD, DbgCtrl*, int, char*);
typedef int (*PLRCTR_ConnectExistDebugger)(DWORD, DbgCtrl*, int);
typedef int (*PLRCTR_DisconnectDebugger)(DbgCtrl*);
typedef int (*PLRCTR_SendCommand)(DbgCtrl*, const char*, char**, DWORD*);
typedef int (*PLRCTR_PollEvent)(DbgCtrl*, int *, char**, DWORD*);
typedef int (*PLRCTR_PollLog)(DbgCtrl*, char**, DWORD*);

typedef struct {
	DWORD id;	/* ID of PARTNER device to which debugger software is connected to */
	DWORD pid;	/* Process ID of debugger software */
} DbgList;
typedef int (*PLRCTR_getDebuggerList)(DbgList**);
typedef struct {
	DWORD id;	/* ID of PARTNER device to which debugger software is connected to */
	DWORD pid;	/* Process ID of debugger software */
	DWORD kind;	/* Debugger software type: DEBUGGER_CTRS or DEBUGGER_CTRJ */
	DWORD corenum;	/* Connected core number (JTAG only) */
} DbgList2;
typedef int (*PLRCTR_getDebuggerList2)(DbgList2**);
typedef int (*PLRCTR_terminateDebugger)(DWORD);


#if     defined(_MFC_VER) && defined(_AFXDLL)
#define LOADLIBRARY AfxLoadLibrary
#define FREELIBRARY AfxFreeLibrary
#else
#define LOADLIBRARY LoadLibrary
#define FREELIBRARY FreeLibrary
#endif

class CLoadRun {
protected:
	HINSTANCE m_hDLL;

	PLRCTR_id2serialno m_pId2serialno;
	PLRCTR_serialno2id m_pSerialno2id;
	PLRCTR_getDefaultBinDir m_pGetDefaultBinDir;
	PLRCTR_getDefaultCfgDir m_pGetDefaultCfgDir;
	PLRCTR_setDir m_pSetDir;
	PLRCTR_setAllocator m_pSetAllocator;
	PLRCTR_getList m_pGetList;
	PLRCTR_ConnectDebuggerEx m_pConnectDebuggerEx;
	PLRCTR_ConnectExistDebugger m_pConnectExistDebugger;
	PLRCTR_DisconnectDebugger m_pDisconnectDebugger;
	PLRCTR_SendCommand m_pSendCommand;
	PLRCTR_PollEvent m_pPollEvent;
	PLRCTR_PollLog m_pPollLog;
	PLRCTR_getDebuggerList m_pGetDebuggerList;
	PLRCTR_getDebuggerList2 m_pGetDebuggerList2;
	PLRCTR_terminateDebugger m_pTerminateDebugger;

public:
	CLoadRun()
	{
		m_hDLL = NULL;
		m_pId2serialno = NULL;
		m_pSerialno2id = NULL;
		m_pGetDefaultBinDir = NULL;
		m_pGetDefaultCfgDir = NULL;
		m_pSetDir = NULL;
		m_pSetAllocator = NULL;
		m_pGetList = NULL;
		m_pConnectDebuggerEx = NULL;
		m_pConnectExistDebugger = NULL;
		m_pDisconnectDebugger = NULL;
		m_pSendCommand = NULL;
		m_pPollEvent = NULL;
		m_pPollLog = NULL;
		m_pGetDebuggerList = NULL;
		m_pGetDebuggerList2 = NULL;
		m_pTerminateDebugger = NULL;
	}

	virtual ~CLoadRun()
	{
		Finalize();
	}

	/* Initialization */
	/* Return values: TRUE for success; FALSE for failure. */
	BOOL Initialize()
	{
		if (m_hDLL != NULL) return TRUE;

		TCHAR dirname[MAX_PATH];
		if ((GetEnvironmentVariable(TEXT("KMC_PARTNER_CTRS"), dirname, MAX_PATH) > 0)||
		    (GetEnvironmentVariable(TEXT("KMC_PARTNER_CTRJ"), dirname, MAX_PATH) > 0)) {
			TCHAR temp[MAX_PATH];
			wsprintf(temp, TEXT("%s\\BIN\\%s"), dirname, TEXT("loadrun_ctr.dll"));
			m_hDLL = LOADLIBRARY(temp);
		} else {
			m_hDLL = LOADLIBRARY(TEXT("loadrun_ctr.dll"));
		}
		if (m_hDLL != NULL) {
			m_pId2serialno = (PLRCTR_id2serialno)GetProcAddress(m_hDLL, "LRCTR_id2serialno");
			m_pSerialno2id = (PLRCTR_serialno2id)GetProcAddress(m_hDLL, "LRCTR_serialno2id");
			m_pGetDefaultBinDir = (PLRCTR_getDefaultBinDir)GetProcAddress(m_hDLL, "LRCTR_getDefaultBinDir");
			m_pGetDefaultCfgDir = (PLRCTR_getDefaultCfgDir)GetProcAddress(m_hDLL, "LRCTR_getDefaultCfgDir");
			m_pSetDir = (PLRCTR_setDir)GetProcAddress(m_hDLL, "LRCTR_setDir");
			m_pSetAllocator = (PLRCTR_setAllocator)GetProcAddress(m_hDLL, "LRCTR_setAllocator");
			m_pGetList = (PLRCTR_getList)GetProcAddress(m_hDLL, "LRCTR_getList");
			m_pConnectDebuggerEx = (PLRCTR_ConnectDebuggerEx)GetProcAddress(m_hDLL, "LRCTR_ConnectDebuggerEx");
			m_pConnectExistDebugger = (PLRCTR_ConnectExistDebugger)GetProcAddress(m_hDLL, "LRCTR_ConnectExistDebugger");
			m_pDisconnectDebugger = (PLRCTR_DisconnectDebugger)GetProcAddress(m_hDLL, "LRCTR_DisconnectDebugger");
			m_pSendCommand = (PLRCTR_SendCommand)GetProcAddress(m_hDLL, "LRCTR_SendCommand");
			m_pPollEvent = (PLRCTR_PollEvent)GetProcAddress(m_hDLL, "LRCTR_PollEvent");
			m_pPollLog = (PLRCTR_PollLog)GetProcAddress(m_hDLL, "LRCTR_PollLog");
			m_pGetDebuggerList = (PLRCTR_getDebuggerList)GetProcAddress(m_hDLL, "LRCTR_getDebuggerList");
			m_pGetDebuggerList2 = (PLRCTR_getDebuggerList2)GetProcAddress(m_hDLL, "LRCTR_getDebuggerList2");
			m_pTerminateDebugger = (PLRCTR_terminateDebugger)GetProcAddress(m_hDLL, "LRCTR_terminateDebugger");
		}
		return (m_hDLL != NULL);
	}

	/* Finalize. */
	/* Return values: TRUE for success; FALSE for failure. */
	BOOL Finalize()
	{
		if (m_hDLL != NULL) {
			FREELIBRARY(m_hDLL);
		}
		return TRUE;
	}

	/* Converts an ID into a serial string */
	/* Arguments: id: ID to convert; pSerial: converted serial (buffer must be at least 9 bytes long) */
	/* Return values: 0 or greater: Success; Negative value: Failure */
	int Id2SerialNo(DWORD id, char* pSerial)
	{
		return (m_pId2serialno) ? (*m_pId2serialno)(id, pSerial) : -1;
	}

	/* Converts a serial string into an ID */
	/* Return values: 0 or greater: Success; Negative value: Failure */
	int SerialNo2Id(const char* pSerial, DWORD *pId)
	{
		return (m_pSerialno2id) ? (*m_pSerialno2id)(pSerial, pId) : -1;
	}

	/* Get the BIN directory where PARTNER is installed */
	/* Arguments: kind: Debugger type (DEBUGGER_XXX) */
	/* Return values: Other than NULL: Directory string (NULL terminated); NULL: Failure */
	const char* GetDefaultBinDir(int kind)
	{
		return (m_pGetDefaultBinDir) ? (*m_pGetDefaultBinDir)(kind) : NULL;
	}

	/* Get PARTNER's default project directory */
	/* Arguments: kind: Debugger type (DEBUGGER_XXX) */
	/* Return values: Other than NULL: Directory string (NULL terminated); NULL: Failure */
	const char* GetDefaultCfgDir(int kind)
	{
		return (m_pGetDefaultCfgDir) ? (*m_pGetDefaultCfgDir)(kind) : NULL;
	}

	/* Set type and directory of PARTER to connect */
	/* Arguments: pBinDir: BIN directory of PARTNER; pCfgDir: Directory of project to start; kind: debugger type (DEBUGGER_XXX) */
	/* Return values: 0 or greater: Success; Negative value: Failure */
	int SetPartnerDir(const char* pBinDir, const char* pCfgDir, int kind)
	{
		return (m_pSetDir) ? (*m_pSetDir)(pBinDir, pCfgDir, kind) : -1;
	}

	/* Set type of PARTER to connect to default */
	/* Arguments: kind: Debugger type (DEBUGGER_XXX) */
	/* Return values: 0 or greater: Success; Negative value: Failure */
	int SetDefaultDir(int kind)
	{
		return SetPartnerDir(GetDefaultBinDir(kind), GetDefaultCfgDir(kind), kind);
	}

	/* Set memory allocation API */
	/* Configure memory allocation function to use with GetDebuggerList, SendCommand, PollEvent, and PollLog */
	/* Arguments: pAllocator:alloc function */
	/* Return values: 0 or greater: Success; Negative value: Failure */
	/* If this function has not been called, or pAllocator is NULL: */
	/* HeapAlloc(GetProcessHeap(),0, size) is called inside the DLL.*/
	int SetAllocator(void* (*pAllocator)(size_t))
	{
		return (m_pSetAllocator) ? (*m_pSetAllocator)(pAllocator) : -1;
	}

	/* Get list of devices we can connect to. */
	/* Arguments: Pointer to the list we obtained */
	/* Return values: Number of debuggers we found; Negative value means failure */
	int GetDebuggerList(DWORD **pIds)
	{
		return (m_pGetList) ? (*m_pGetList)(pIds) : -1;
	}

#define CLRCTR_ATTR_HIDE		(0x00000001)
#define CLRCTR_ATTR_SYSTRAY	(0x00000002)
#define CLRCTR_ATTR_NOCMD	(0x00000004)
#define CLRCTR_ATTR_NOLOG	(0x00000008)
#define CLRCTR_ATTR_NOEVT	(0x00000010)
	/* Start new debugger instance and connect */
	/* Arguments: id: Debugger to connect (0 means not specified) */
	/*       pCtrl: connection management structure / attr: startup option / extra: additional startup option */
	/* Return values: 0 or greater: Success; Negative value: Failure */
	int ConnectDebugger(DWORD id, DbgCtrl* pCtrl, int attr, char* extra=NULL)
	{
		return (m_pConnectDebuggerEx) ? (*m_pConnectDebuggerEx)(id, pCtrl, attr, extra) : -1;
	}

	/* Connect to debugger that is already running */
	/* Arguments: pid: Process ID of debugger to connect to */
	/*       pCtrl: connection management structure / attr: startup option (CLRCTR_ATTR_NOxxx only) */
	/* Return values: 0 or greater: Success; Negative value: Failure */
	int ConnectExistDebugger(DWORD pid, DbgCtrl* pCtrl, int attr)
	{
		return (m_pConnectExistDebugger) ? (*m_pConnectExistDebugger)(pid, pCtrl, attr) : -1;
	}

	/* Disconnect debugger */
	/* Arguments: pCtrl: Connection management structure */
	/* Return values: 0 or greater: Success; Negative value: Failure */
	int DisconnectDebugger(DbgCtrl *pCtrl)
	{
		return (m_pDisconnectDebugger) ? (*m_pDisconnectDebugger)(pCtrl) : -1;
	}

	/* Send command to debugger */
	/* Arguments: pCtrl: Connection management structure / pCmd: Command string to send */
	/*       pResTxt: Response output / pLength: Response output string length*/
	/* Return values: Negative value: failure; 0: Success; Positive value: command error */
	int SendCommand(DbgCtrl* pCtrl, const char *pCmd, char **pResTxt, DWORD *pLength)
	{
		return (m_pSendCommand) ? (*m_pSendCommand)(pCtrl, pCmd, pResTxt, pLength) : -1;
	}

#define CLRCTR_EVENT_EXIT	(0)
#define CLRCTR_EVENT_BP_SET	(1)
#define CLRCTR_EVENT_BP_DISABLE	(2)
#define CLRCTR_EVENT_BP_ENABLE	(3)
#define CLRCTR_EVENT_BP_CLEAR	(4)
	/* Check event notification from debugger */
	/* Arguments: pCtrl: Connection management structure / pEvent: Type of event received */
	/*       pResTxt: Response output / pLength: Response output string length*/
	/* Return values: Negative value: failure; 0: no data; Positive value: has data */
	int PollEvent(DbgCtrl* pCtrl, int *pEvent, char** pEvtTxt, DWORD *pLength)
	{
		return (m_pPollEvent) ? (*m_pPollEvent)(pCtrl, pEvent, pEvtTxt, pLength) : -1;
	}

	/* Check log output from application*/
	/* Arguments: pCtrl: Connection management structure  */
	/*       pLogTxt: Log output / pLength: Log output string length*/
	int PollLog(DbgCtrl* pCtrl, char** pLogTxt, DWORD *pLength)
	/* Return values: Negative value: failure; 0: no data; Log data string length */
	{
		return (m_pPollLog) ? (*m_pPollLog)(pCtrl, pLogTxt, pLength) : -1;
	}

	/* Get list of currently running debugger software lists */
	/* Arguments: pList: The list we obtained */
	/* Return values: Negative value: Failure; 0 or greater: Number of running debugger software lists */
	int GetDebuggerSoftwareList(DbgList** pList)
	{
		return (m_pGetDebuggerList) ? (*m_pGetDebuggerList)(pList) : -1;
	}

	/* Get list of currently running debugger software instances (extended) */
	/* Arguments: pList: The list we obtained */
	/* Return values: Negative value: Failure; 0 or greater: Number of running debugger software lists */
	int GetDebuggerSoftwareList(DbgList2** pList)
	{
		return (m_pGetDebuggerList2) ? (*m_pGetDebuggerList2)(pList) : -1;
	}

	/* Exit running debugger software */
	/* Arguments: pid: Process ID of debugger to exit */
	/* Return values: Negative value: Failure; 0 or greater: Success */
	int TerminateDebuggerSoftware(DWORD pid)
	{
		return (m_pTerminateDebugger) ? (*m_pTerminateDebugger)(pid) : -1;
	}
};

}
}
}

#endif /* _C_LOADRUN_CTR_ */
